/*--------------------------------------------------------------------
 * FICHERO:  Hiperipe.c
 * OBJETIVO: Leer y grabar disquetes completos (por sectores)
 *           a ficheros
 * AUTOR:    Pedro Reina
 * FECHA:    V.6.2.1998
 *------------------------------------------------------------------*/

/*--------------------------------------------------------------------
 * Ficheros de cabecera
 *------------------------------------------------------------------*/

#include <Olimpo.h>   /*  Versin 2.0  */
#include <bios.h>     /*  biosdisk()   */

/*--------------------------------------------------------------------
 * Declaracin de funciones
 *------------------------------------------------------------------*/

void Explica();
void ExplicaObjetivo();
void ExplicaManejo();
void ExplicaNombre();
void ExplicaAutor();

void   Trabajo();
void   LeeDisquete();
void   EscribeDisquete();
logico ProcesaPista();
logico AvisoImportante();
region PreparaGrafico();
void   MuestraGrafico();

int    PideTarea();
int    PideDisquetera();
int    PideDensidad();
cadena EligeNombre();
int    ComparaCadena();
cadena PideNombre();

void Opciones();

/*--------------------------------------------------------------------
 * Definicin de macros constantes
 *------------------------------------------------------------------*/

#define LONG_NOMBRE    8
#define LONG_SECTOR  512

#define EXTENSION  "img"

/* Posibles tareas */
#define LEER     2
#define ESCRIBIR 3

/* Posibles disqueteras */
#define DISQUETERA_A  0
#define DISQUETERA_B  1

/* Posibles densidades */
#define DOBLE  0
#define ALTA   1

/* Smbolos de mensaje al usuario */
#define PREPARADO  '*'
#define BIEN       ''
#define ERROR      'E'

/*--------------------------------------------------------------------
 * Variables globales
 *------------------------------------------------------------------*/

logico HayQueOrdenar=NO;

/*--------------------------------------------------------------------
 * Definicin de macros funciones
 *------------------------------------------------------------------*/

#define LimpiaZonaPrincipal()  Pan_Borra(NEGRO,4,0,21,79)
#define LimpiaZonaTrabajo()    Pan_Borra(NEGRO,11,0,21,79)

/*--------------------------------------------------------------------
 * MACRO:    ResetControladora()
 * OBJETIVO: Hacer un reset en la controladora de disquetes
 *------------------------------------------------------------------*/
#define ResetControladora()  biosdisk(0,0,0,0,0,0,0)

/*--------------------------------------------------------------------
 * Programa principal
 *------------------------------------------------------------------*/
void main (void)
  {
  enum { SALIDA_ESC, SALIDA, EXPLICA, TRABAJO, OPCIONES };

  static cadena Principal[] = {">Salida", "E>xplica", ">Trabajo",
                               ">Opciones", NIL };
  entero Opcion=TRABAJO;
  logico Sigue=SI;

  Pan_Define (PAN_TEXTO);
  Prg_Presenta ( "Hiperipe", "1.0", "Pedro Reina", "Febrero 1998" );
  Cdr_Caja (CDR_SIMPLE,1,0,3,79,NEGRO,VERDE);

  while ( Sigue )
    {
    Opcion = Men_Horizontal (2,2,77,Principal,Opcion);
    switch ( Opcion )
      {
      case SALIDA_ESC:
      case SALIDA:
           if ( Usr_Consulta ("Quieres terminar el programa?") )
             { Sigue = NO; }
           Opcion = SALIDA;
           break;
      case EXPLICA:       Explica();       break;
      case TRABAJO:       Trabajo();       break;
      case OPCIONES:      Opciones();      break;
      }
    LimpiaZonaPrincipal();
    }

  Pan_Cierra();
  }

/*--------------------------------------------------------------------
 * Definicin de funciones
 *------------------------------------------------------------------*/

/*--------------------------------------------------------------------
 * FUNCION:  Explica()
 * OBJETIVO: Dirigir las explicaciones
 *------------------------------------------------------------------*/
void Explica()
  {
  enum { SALIDA_ESC, OBJETIVO, MANEJO, NOMBREPROG, AUTOR };
  static octeto MaxOpcion = 4;

  static cadena MenuExplica[] = { ">Objetivo del programa",
                                  ">Manejo de las opciones",
                                  ">Nombre del programa",
                                  ">Contactar con el autor",
                                  NIL };

  static cadena Presentacion[] = {
    "Puedes pedir explicacin sobre diferentes aspectos",
    "de este programa. Elige el que desees y pulsa ESC",
    "cuando hayas terminado.",
    NIL };

  logico Sigue = SI;
  entero Opcion = 1, i;

  while ( Sigue )
    {
    LimpiaZonaPrincipal();
    Pan_Color (NEGRO,BLANCO);
    for ( i=0 ; Presentacion[i] ; i++ )
      { Pan_PonTexto (6+i,15,Presentacion[i]); }

    Opcion = Men_Vertical (11,29,14,52,MenuExplica,Opcion);
    LimpiaZonaPrincipal();
    switch ( Opcion )
      {
      case SALIDA_ESC:  Sigue = NO;         break;
      case OBJETIVO:    ExplicaObjetivo();  break;
      case MANEJO:      ExplicaManejo();    break;
      case NOMBREPROG:  ExplicaNombre();    break;
      case AUTOR:       ExplicaAutor();     break;
      }
    if ( Opcion )  { Usr_PulsaUnaTecla(""); }
    Opcion++;
    if ( Opcion > MaxOpcion )  { Opcion = 1; }
    }
  }

/*--------------------------------------------------------------------
 * FUNCION:  ExplicaObjetivo()
 * OBJETIVO: Explicar el objetivo del programa
 *------------------------------------------------------------------*/
void ExplicaObjetivo()
  {
  static cadena Mensaje[] = {
    "Este programa permite leer disquetes de 720 K y de 1.44 M, en",
    "cualquier formato, a un fichero y tambin escribir un disquete",
    "a partir de un fichero.",
    "",
    "Esto permite hacer fcilmente copias de seguridad sobre unidades",
    "MS-DOS de disquetes de otros sistemas operativos. Los ficheros",
    "creados tienen extensin 'img' y pueden ser comprimidos para que",
    "se almacenen en menos espacio.",
    NIL };

  octeto i;

  Pan_Color (NEGRO,BLANCO);
  for ( i=0 ; Mensaje[i] ; i++ )
    { Pan_PonTexto (6+i,6,Mensaje[i]); }
  }

/*--------------------------------------------------------------------
 * FUNCION:  ExplicaManejo()
 * OBJETIVO: Explicar cmo se maneja el programa
 *------------------------------------------------------------------*/
void ExplicaManejo()
  {
  static cadena Mensaje[] = {
    "La opcin Salida sirve para concluir le ejecucin del programa.",
    "",
    "La opcin Explica te est sirviendo para comprender cmo usarlo.",
    "",
    "La opcin Trabajo es la ms importante, ya que dirige la lectura",
    "y escritura de disquetes. Primero debes elegir la tarea que vas a",
    "realizar, la disquetera que vas a usar, la densidad del disquete",
    "y el nombre del fichero. Por ltimo, usa la opcin Empezar para",
    "realizar la tarea.",
    "",
    "En Opciones se pueden definir dos cosas: si el sonido est",
    "encendido o apagado y si se desea ver por orden alfabtico",
    "los ficheros cuando se va a leer uno.",
    NIL };

  octeto i;

  Pan_Color (NEGRO,BLANCO);
  for ( i=0 ; Mensaje[i] ; i++ )
    { Pan_PonTexto (5+i,4,Mensaje[i]); }
  }

/*--------------------------------------------------------------------
 * FUNCION:  ExplicaNombre()
 * OBJETIVO: Explicar por qu el programa se llama as
 *------------------------------------------------------------------*/
void ExplicaNombre()
  {
  static cadena Mensaje[] = {
    "Hiperipe es una de las danaides, la que cas con Hipocoristes.",
    "Las danaides fueron condenadas a llenar eternamente un tonel sin",
    "fondo, como castigo por haber asesinado a sus esposos en la noche",
    "de bodas.",
    NIL };

  octeto i;

  Pan_Color (NEGRO,BLANCO);
  for ( i=0 ; Mensaje[i] ; i++ )
    { Pan_PonTexto (6+i,6,Mensaje[i]); }
  }

/*--------------------------------------------------------------------
 * FUNCION:  ExplicaAutor()
 * OBJETIVO: Explicar quin es el autor y cmo contactar con l
 *------------------------------------------------------------------*/
void ExplicaAutor()
  {
  static cadena Mensaje[] = {
    "Este programa es de Dominio Pblico. Lo puedes usar, distribuir",
    "y modificar como desees.",
    "",
    "Ha sido escrito en C usando el sistema de programacin Olimpo;",
    "si no dispones de la versin 2.0 de Olimpo, puedes cargarla",
    "directamente desde mi sede web o pedrmela si no dispones de",
    "acceso a Internet.",
    "",
    "Si tienes cualquier duda o consulta, cuntamela e intentar ayudarte.",
    "",
    "Mis datos:",
    "",
    "Pedro Reina",
    "c/ Marquesa de Argeso, 4      Telfono: 91 565 17 59",
    "28019 Madrid                   Correo electrnico: pedro@anit.es",
    "Espaa                         Web: www.anit.es/pedro",
    NIL };

  octeto i;

  Pan_Color (NEGRO,BLANCO);
  for ( i=0 ; Mensaje[i] ; i++ )
    { Pan_PonTexto (5+i,5,Mensaje[i]); }
  }

/*--------------------------------------------------------------------
 * FUNCION:  Trabajo()
 * OBJETIVO: Presentar y manejar las opciones principales del programa
 * ENTRADAS: Ninguna
 * SALIDAS:  Ninguna
 *------------------------------------------------------------------*/
void Trabajo()
  {
  enum { SALIDA_ESC, TAREA, DISQUETERA, DENSIDAD, FICHERO, EMPEZAR };
  static cadena MenuTrab[] = {">Tarea", ">Disquetera", "De>nsidad",
                              ">Fichero", ">Empezar", NIL };

  static caracter Nombre[LONG_NOMBRE+1] = "";
  static int Tarea = LEER;
  static int Disquetera = DISQUETERA_A;
  static int Densidad = ALTA;

  cadena Aux;
  region Region;
  entero Opcion = FICHERO;
  logico Sigue = SI, Refresca = SI;

  if ( Region = Reg_Crea (4,25,10,53) )
    {
    Reg_Borde (Region,CDR_SIMPLE,NEGRO,VERDE);
    Reg_Color (Region,NEGRO,VERDE);
    while ( Sigue )
      {
      if ( Refresca )
        {
        Pan_Borra (NEGRO,Reg_FilSup(Region)+1, Reg_ColIzq(Region)+15,
                         Reg_FilInf(Region)-1, Reg_ColDer(Region)-1);
        if ( Tarea == LEER )  { Reg_PonTexto (Region,1,15,"Leer"); }
        else                  { Reg_PonTexto (Region,1,15,"Escribir"); }
        if ( Disquetera == DISQUETERA_A )  { Reg_PonTexto (Region,2,15,"A:"); }
        else                               { Reg_PonTexto (Region,2,15,"B:"); }
        if ( Densidad == DOBLE )  { Reg_PonTexto (Region,3,15,"Doble"); }
        else                      { Reg_PonTexto (Region,3,15,"Alta"); }
        if ( Nombre[0] )
          {
          Reg_PonTexto (Region,4,15,Nombre);
          Pan_Caracter ('.');
          Pan_Texto (EXTENSION);
          }
        Refresca = NO;
        }

      Opcion = Men_Vertical (Reg_FilSup(Region)+1, Reg_ColIzq(Region)+1,
                             Reg_FilInf(Region)-1, Reg_ColIzq(Region)+14,
                             MenuTrab, Opcion);
      switch ( Opcion )
        {
        case SALIDA_ESC:  Sigue = NO;  break;

        case TAREA:
          Tarea = PideTarea (Tarea);
          Refresca = SI;
          break;

        case DISQUETERA:
          Disquetera = PideDisquetera (Disquetera);
          Refresca = SI;
          break;

        case DENSIDAD:
          Densidad = PideDensidad (Densidad);
          Refresca = SI;
          break;

        case FICHERO:
           if ( Tarea == ESCRIBIR )
             {
             Aux = EligeNombre(Nombre);
             if ( Tec_Ultima() == TEC_ENTER )
               {
               Cad_Copia (Nombre,Aux);
               Refresca = SI;
               }
             Cad_Destruye (Aux);
             }
           else  /* Tarea == LEER */
             {
             Aux = PideNombre (Nombre);
             if ( Tec_Ultima() != TEC_ESC )
               {
               Cad_Copia (Nombre,Aux);
               Refresca = SI;
               }
             Cad_Destruye (Aux);
             }
           break;

        case EMPEZAR:
          if ( !Nombre[0] )
            { Usr_Avisa ("Debe haber un nombre de fichero"); }
          else
            {
            if ( Tarea == LEER )
              { LeeDisquete (Disquetera, Densidad, Nombre); }
            else
              {
              if ( AvisoImportante() )
                { EscribeDisquete (Disquetera, Densidad, Nombre); }
              }
            }
          break;
        }
      LimpiaZonaTrabajo();
      }
    Reg_Destruye (Region);
    }
  else  { Usr_Avisa ("Falta memoria"); }
  }

/*--------------------------------------------------------------------
 * FUNCION:  LeeDisquete()
 * OBJETIVO: Leer un disquete y guardarlo en un fichero
 * ENTRADAS: La disquetera en que est el disquete, la densidad y el
 *           nombre del fichero
 * SALIDAS:  Ninguna
 *------------------------------------------------------------------*/
void LeeDisquete (Disquetera, Densidad, Nombre)
int Disquetera, Densidad;
cadena Nombre;
  {
  static octeto Info[18*LONG_SECTOR];
  fichero Fichero;
  region  Panel;
  cadena  Completo;
  logico  Error=NO, ErrorFichero, ErrorDisquete;
  int     NumeroSectores, Cara, Pista;

  if ( Densidad == DOBLE )  { NumeroSectores =  9; }
  else                      { NumeroSectores = 18; }

  Completo = Fch_Nombre (Nombre, EXTENSION);

  if ( Fichero = Fch_AbreGrabar (Completo, FCH_BINARIO) )
    {
    Usr_Informa ("Leyendo disquete");
    Panel = PreparaGrafico();

    for ( Pista = 0 ; Pista < 80 && !Error ; Pista++ )
      {
      for ( Cara = 0 ; Cara < 2 && !Error ; Cara++ )
        {
        MuestraGrafico (Panel, Pista, Cara, PREPARADO);
        ErrorDisquete = ProcesaPista (LEER, Disquetera, Cara, Pista,
                                      NumeroSectores, Info);
        if ( ErrorDisquete )  { MuestraGrafico (Panel, Pista, Cara, ERROR); }
        else                  { MuestraGrafico (Panel, Pista, Cara, BIEN); }
        ErrorFichero = !Fch_EscribeOcteto (Fichero, Info,
                                           NumeroSectores*LONG_SECTOR);
        Error = ErrorDisquete || ErrorFichero;
        }
      }

    Reg_Destruye (Panel);
    Fch_Cierra (Fichero);

    if ( ErrorDisquete )
      { Usr_Avisa ("Ha habido un error leyendo el disquete"); }
    if ( ErrorFichero )
      { Usr_Avisa ("Ha habido un error escribiendo el fichero"); }
    if ( !Error )
      {
      Son_Bien();
      Usr_PulsaUnaTecla ("Lectura completada con xito");
      }
    }

  Cad_Destruye (Completo);
  }

/*--------------------------------------------------------------------
 * FUNCION:  EscribeDisquete()
 * OBJETIVO: Escribir un disquete con los datos de un fichero
 * ENTRADAS: La disquetera en que est el disquete, la densidad y el
 *           nombre del fichero
 * SALIDAS:  Ninguna
 *------------------------------------------------------------------*/
void EscribeDisquete (Disquetera, Densidad, Nombre)
int Disquetera, Densidad;
cadena Nombre;
  {
  static octeto Info[18*LONG_SECTOR];
  fichero Fichero;
  region  Panel;
  cadena  Completo;
  logico  Error=NO, ErrorFichero, ErrorDisquete;
  int     NumeroSectores, Cara, Pista;

  if ( Densidad == DOBLE )  { NumeroSectores =  9; }
  else                      { NumeroSectores = 18; }

  Completo = Fch_Nombre (Nombre, EXTENSION);

  if ( Fichero = Fch_AbreLeer (Completo, FCH_BINARIO) )
    {
    Usr_Informa ("Escribiendo disquete");
    Panel = PreparaGrafico();

    for ( Pista = 0 ; Pista < 80 && !Error ; Pista++ )
      {
      for ( Cara = 0 ; Cara < 2 && !Error ; Cara++ )
        {
        MuestraGrafico (Panel, Pista, Cara, PREPARADO);
        ErrorFichero = !Fch_LeeOcteto (Fichero, Info,
                                       NumeroSectores*LONG_SECTOR);
        ErrorDisquete = ProcesaPista (ESCRIBIR, Disquetera, Cara, Pista,
                                      NumeroSectores, Info);
        if ( ErrorDisquete )  { MuestraGrafico (Panel, Pista, Cara, ERROR); }
        else                  { MuestraGrafico (Panel, Pista, Cara, BIEN); }
        Error = ErrorDisquete || ErrorFichero;
        }
      }

    Reg_Destruye (Panel);
    Fch_Cierra (Fichero);

    if ( ErrorDisquete )
      { Usr_Avisa ("Ha habido un error escribiendo en el disquete"); }
    if ( ErrorFichero )
      { Usr_Avisa ("Ha habido un error leyendo el fichero"); }
    if ( !Error )
      {
      Son_Bien();
      Usr_PulsaUnaTecla ("Escritura completada con xito");
      }
    }

  Cad_Destruye (Completo);
  }

/*--------------------------------------------------------------------
 * FUNCION:  ProcesaPista()
 * OBJETIVO: Leer o escribir una pista completa
 * ENTRADAS: La tarea que hay que realizar, la disquetera en que est
 *           el disquete, la cara, la pista, y la zona de memoria donde
 *           se encuentran o se dejan los datos
 * SALIDAS:  Lgica, que indica si ha habido error
 * ALGORITMO: Se intenta 4 veces cada operacin para dar tiempo al motor
 *            a encender y al MS-DOS a enterarse
 *------------------------------------------------------------------*/
logico ProcesaPista (Tarea, Disquetera, Cara, Pista, NumSect, Memoria)
int Tarea, Disquetera, Cara, Pista, NumSect;
memoria Memoria;
  {
  menudo Intento;
  logico Respuesta;
  int    Error=1;

  for ( Intento = 1 ; Intento<5 && Error ; Intento++ )
    {
    Error = biosdisk (Tarea, Disquetera, Cara, Pista, 1, NumSect,
                      (char *)Memoria);
    if ( Error )  { ResetControladora(); }
    }

  if ( Error )  { Respuesta = SI; }
  else          { Respuesta = NO; }

  return ( Respuesta );
  }

/*--------------------------------------------------------------------
 * FUNCION:  AvisoImportante()
 * OBJETIVO: Avisar al usuario de que el proceso de escritura es
 *           peligroso y pedir confirmacin para seguir
 * ENTRADAS: Ninguna
 * SALIDAS:  Lgica, que indica si hay que seguir adelante
 *------------------------------------------------------------------*/
logico AvisoImportante()
  {
  region Region;

  Region = Reg_Crea (12,16,20,64);
  Reg_Limpia (Region, ROJO);
  Reg_Borde (Region,CDR_DOBLE,ROJO,BLANCO);
  Reg_Color (Region, ROJO, BLANCO);

  Reg_TextoCentrado (Region, "Aviso importante", 2);
  Reg_TextoCentrado (Region, "El proceso de escritura de un", 4);
  Reg_TextoCentrado (Region, "disquete destruye todos sus", 5);
  Reg_TextoCentrado (Region, "datos y es irreversible", 6);

  Reg_Destruye (Region);

  return ( Usr_Consulta ("Ests seguro de querer seguir?") );
  }

/*--------------------------------------------------------------------
 * FUNCION:  PreparaGrafico()
 * OBJETIVO: Crear una region en la que informar al usuario del
 *           proceso de lectura o escritura
 * ENTRADAS: Ninguna
 * SALIDAS:  Una regin
 *------------------------------------------------------------------*/
region PreparaGrafico()
  {
  region Region;

  Region = Reg_Crea (11,14,21,66);
  Reg_Borde (Region,CDR_SIMPLE,NEGRO,VERDE);
  Reg_Color (Region, NEGRO, VERDE);

  Reg_PonTexto (Region,2,3,"Pista: 0....:....1....:....2....:....3....:....");
  Reg_PonTexto (Region,3,3,"Cara 0");
  Reg_PonTexto (Region,4,3,"Cara 1");
  Pan_Borra (BLANCO,Reg_FilSup(Region)+3,Reg_ColIzq(Region)+10,
                    Reg_FilSup(Region)+4,Reg_ColIzq(Region)+49);

  Reg_PonTexto (Region,6,3,"Pista: 4....:....5....:....6....:....7....:....");
  Reg_PonTexto (Region,7,3,"Cara 0");
  Reg_PonTexto (Region,8,3,"Cara 1");
  Pan_Borra (BLANCO,Reg_FilSup(Region)+7,Reg_ColIzq(Region)+10,
                    Reg_FilSup(Region)+8,Reg_ColIzq(Region)+49);

  Reg_Color (Region, BLANCO, ROJO);
  Pan_CursorVisible (NO);

  return ( Region );
  }

/*--------------------------------------------------------------------
 * FUNCION:  MuestraGrafico()
 * OBJETIVO: Mostrar un smbolo en pantalla indicando el estado
 *           de una pista
 * ENTRADAS: La regin donde escribir, la pista, la cara y el smbolo
 * SALIDAS:  Ninguna
 *------------------------------------------------------------------*/
void MuestraGrafico (Region, Pista, Cara, Simbolo)
region   Region;
int      Pista, Cara;
caracter Simbolo;
  {
  octeto Fila, Columna;

  Fila = Pista < 40 ? 3 : 7;
  Fila += Cara;
  Columna = Pista % 40 + 10;

  Reg_PonCar (Region, Fila, Columna, Simbolo);
  }

/*--------------------------------------------------------------------
 * FUNCION:  EligeNombre()
 * OBJETIVO: Elegir un nombre de fichero de entre los posibles
 * ENTRADAS: Un nombre para proponer el primero
 * SALIDAS:  Un nombre, sin directorio ni extensin
 * NOTA:     El nombre hay que destruirlo cuando no sea necesario
 *------------------------------------------------------------------*/
cadena EligeNombre (Nombre)
cadena Nombre;
  {
  cadena   Respuesta, Completo, NombrePropuesto;
  lista    Posible, Examinado;
  entero   Elegido, Propuesto;
  region   Region;
  menu     Eleccion;
  octeto   i;
  logico   Visto;
  caracter Aux[LONG_NOMBRE+5];

  Region = Reg_Crea (11,31,21,48);
  Reg_Borde (Region, CDR_SIMPLE, NEGRO, VERDE);
  Completo = Fch_Nombre("*",EXTENSION);
  Usr_Informa ("Buscando ficheros \"img\"");
  if ( Posible = Fch_ListaNombre (Completo) )
    {
    if ( HayQueOrdenar )
      { Posible = Lis_Ordena (Posible, ComparaCadena); }
    Eleccion = Men_Crea (MEN_VERTICAL,
                         Reg_FilSup(Region)+1, Reg_ColIzq(Region)+2,
                         Reg_FilInf(Region)-1, Reg_ColDer(Region)-2,
                         Lis_Total(Posible), MEN_LISTA, (memoria)Posible);

    /* Buscamos Nombre en Posible */
    Visto = NO;
    NombrePropuesto = Cad_Mayus (Fch_Nombre (Nombre, EXTENSION));
    for ( Examinado = Posible , Propuesto = 1 ; 
          Examinado && !Visto ; 
          Examinado = Lis_Siguiente (Examinado) , Propuesto++ )
      {
      Visto = Cad_Igual (Cad_Mayus (Lis_Contenido(Examinado)), 
                         NombrePropuesto);
      }           
    Cad_Destruye (NombrePropuesto);
    Propuesto = Visto ? Propuesto-1 : 1;

    if ( Elegido = Men_Ejecuta (Eleccion, Propuesto,
                   "stos son los ficheros \"img\" del directorio actual. "
                   "Elige uno.") )
      {
      Men_Texto (Eleccion, Elegido, Aux);
      Visto = NO;
      for ( i=Cad_Longitud(Aux)-1 ; i && !Visto ; i-- )
        {
        if ( Aux[i] == Fch_Separador() )
          {
          Aux[i] = NULO;
          Visto = SI;
          }
        }
      Respuesta = Cad_Duplica (Aux);
      }
    else  { Respuesta = Cad_Duplica (""); }

    Lis_Destruye (Posible);
    Men_Destruye (Eleccion);
    }

  else
    {
    Usr_PulsaUnaTecla ("No se ha encontrado ningn fichero \"img\" "
                       "en el directorio actual");
    Respuesta = Cad_Duplica ("");
    }

  Reg_Destruye (Region);
  Cad_Destruye (Completo);

  return ( Respuesta );
  }

/*--------------------------------------------------------------------
 * FUNCION:  ComparaCadena()
 * OBJETIVO: Comparar dos cadenas
 * ENTRADAS: Los punteros a las dos cadenas
 * SALIDAS:  Un int que indica qu cadena es menor
 *------------------------------------------------------------------*/
int ComparaCadena (Cad1,Cad2)
cadena *Cad1, *Cad2;
  {
  return ( Cad_Compara (*Cad1,*Cad2) );
  }

/*--------------------------------------------------------------------
 * FUNCION:  PideTarea()
 * OBJETIVO: Pedir al usuario la tarea que hay que realizar
 * ENTRADAS: La tarea original
 * SALIDAS:  La nueva tarea
 *------------------------------------------------------------------*/
int PideTarea (Tarea)
int Tarea;
  {
  static cadena MenuTarea[] = {">Leer", ">Escribir", NIL };
  entero Opcion;
  region Region;
  int    Respuesta=0;

  if ( Region = Reg_Crea (11,32,16,45) )
    {
    Opcion = Tarea == LEER ? 2 : 1;
    Reg_Borde (Region,CDR_SIMPLE,NEGRO,VERDE);
    Pan_Color (NEGRO, VERDE);
    Reg_TextoCentrado (Region,"Tarea",1);
    Opcion = Men_Vertical (Reg_FilSup(Region)+3, Reg_ColIzq(Region)+2,
                           Reg_FilInf(Region)-1, Reg_ColDer(Region)-1,
                           MenuTarea, Opcion);
    switch ( Opcion )
      {
      case 0: Respuesta = Tarea;     break;
      case 1: Respuesta = LEER;      break;
      case 2: Respuesta = ESCRIBIR;  break;
      }
    Reg_Destruye (Region);
    }
  else  { Usr_Avisa ("Falta memoria"); }

  return ( Respuesta );
  }

/*--------------------------------------------------------------------
 * FUNCION:  PideDisquetera()
 * OBJETIVO: Pedir al usuario la disquetera que hay que usar
 * ENTRADAS: La disquetera original
 * SALIDAS:  La nueva disquetera
 *------------------------------------------------------------------*/
int PideDisquetera (Disquetera)
int Disquetera;
  {
  static cadena MenuDisquetera[] = {"A:", "B:", NIL };
  entero Opcion;
  region Region;
  int    Respuesta=0;

  if ( Region = Reg_Crea (11,32,16,45) )
    {
    Opcion = Disquetera == DISQUETERA_A ? 2 : 1;
    Reg_Borde (Region,CDR_SIMPLE,NEGRO,VERDE);
    Pan_Color (NEGRO, VERDE);
    Reg_TextoCentrado (Region,"Disquetera",1);
    Opcion = Men_Vertical (Reg_FilSup(Region)+3, Reg_ColIzq(Region)+5,
                           Reg_FilInf(Region)-1, Reg_ColDer(Region)-1,
                           MenuDisquetera, Opcion);
    switch ( Opcion )
      {
      case 0: Respuesta = Disquetera;    break;
      case 1: Respuesta = DISQUETERA_A;  break;
      case 2: Respuesta = DISQUETERA_B;  break;
      }
    Reg_Destruye (Region);
    }
  else  { Usr_Avisa ("Falta memoria"); }

  return ( Respuesta );
  }

/*--------------------------------------------------------------------
 * FUNCION:  PideDensidad()
 * OBJETIVO: Pedir al usuario la densidad que hay que usar
 * ENTRADAS: La densidad original
 * SALIDAS:  La nueva densidad
 *------------------------------------------------------------------*/
int PideDensidad (Densidad)
int Densidad;
  {
  static cadena MenuDensidad[] = {">Doble", ">Alta", NIL };
  entero Opcion;
  region Region;
  int    Respuesta=0;

  if ( Region = Reg_Crea (11,32,16,43) )
    {
    Opcion = Densidad == DOBLE ? 2 : 1;
    Reg_Borde (Region,CDR_SIMPLE,NEGRO,VERDE);
    Pan_Color (NEGRO, VERDE);
    Reg_TextoCentrado (Region,"Densidad",1);
    Opcion = Men_Vertical (Reg_FilSup(Region)+3, Reg_ColIzq(Region)+2,
                           Reg_FilInf(Region)-1, Reg_ColDer(Region)-1,
                           MenuDensidad, Opcion);
    switch ( Opcion )
      {
      case 0: Respuesta = Densidad; break;
      case 1: Respuesta = DOBLE;    break;
      case 2: Respuesta = ALTA;     break;
      }
    Reg_Destruye (Region);
    }
  else  { Usr_Avisa ("Falta memoria"); }

  return ( Respuesta );
  }

/*--------------------------------------------------------------------
 * FUNCION:  PideNombre()
 * OBJETIVO: Pedir al usuario un nombre
 * ENTRADAS: El nombre propuesto
 * SALIDAS:  El nuevo nombre
 *------------------------------------------------------------------*/
cadena PideNombre (Nombre)
cadena Nombre;
  {
  region Region;
  cadena Respuesta;

  Region = Reg_Crea (11,27,17,51);
  Reg_Borde (Region,CDR_SIMPLE,NEGRO,VERDE);
  Pan_Color (NEGRO,VERDE);
  Reg_TextoCentrado (Region,"Nombre del fichero",2);
  Respuesta = Usr_Texto (Nombre, LONG_NOMBRE,
                         Reg_FilSup(Region)+4,Reg_ColIzq(Region)+8,
                         ROJO, BLANCO);

  return ( Respuesta );
  }

/*--------------------------------------------------------------------
 * FUNCION:  Opciones()
 * OBJETIVO: Seleccionar las opciones del programa
 * ENTRADAS: Ninguna
 * SALIDAS:  Las variables globales pueden modificarse
 *------------------------------------------------------------------*/
void Opciones()
  {
  enum { SALIDA_ESC, SONIDO, ORDEN };
  static cadena MenuOpc[] = {">Sonido", ">Orden", NIL };
  entero Opcion = SONIDO;
  logico Sigue = SI;
  region Region;

  if ( Region = Reg_Crea (4,59,7,79) )
    {
    Reg_Borde (Region,CDR_SIMPLE,NEGRO,VERDE);
    while ( Sigue )
      {
      Pan_Color (NEGRO, VERDE);
      Reg_Cursor (Region,1,10);
      if ( Son_Estado() )  { Pan_Texto ("Encendido"); }
      else                 { Pan_Texto ("Apagado  "); }

      Reg_Cursor (Region,2,10);
      if ( HayQueOrdenar )  { Pan_Texto ("S"); }
      else                  { Pan_Texto ("No"); }

      Opcion = Men_Vertical (Reg_FilSup(Region)+1, Reg_ColIzq(Region)+1,
                             Reg_FilInf(Region)-1, Reg_ColIzq(Region)+9,
                             MenuOpc, Opcion);
      switch ( Opcion )
        {
        case SALIDA_ESC: Sigue = NO;                      break;
        case SONIDO:     Son_Cambia();                    break;
        case ORDEN:      HayQueOrdenar = !HayQueOrdenar;  break;
        }
      }
    Reg_Destruye (Region);
    }
  else  { Usr_Avisa ("Falta memoria"); }
  }